#include "wrapper.h"
#include "sockio.h"

int new_sock(void) 
{
	static int p_num = IPPROTO_TCP; /* set to tcp protocol # on first call */
	return Socket(AF_INET, SOCK_STREAM, p_num);
}


int bind_sock(int sockd, int service)
{
	int ret = 0;
	struct sockaddr_in saddr;
	memset((char *)&saddr,0,sizeof(saddr));
	saddr.sin_family = AF_INET;
	saddr.sin_port = htons((u_short)service);
	saddr.sin_addr.s_addr = INADDR_ANY;
#ifndef __KERNEL__
bind_sock_restart:
#endif
	if ((ret = Bind(sockd,(struct sockaddr *)&saddr,sizeof(saddr))) < 0) {
#ifndef __KERNEL__
		if(errno == EINTR)
			goto bind_sock_restart;
#endif
		return ret;
	}
	return sockd;
}

int connect_sock(int sockd, struct sockaddr *saddr, unsigned int size)
{
	int ret = 0;
#ifndef __KERNEL__
connect_sock_restart:
#endif
	if ((ret = Connect(sockd,(struct sockaddr *)saddr,size)) < 0) {
#ifndef __KERNEL__
		if(errno == EINTR)
			goto connect_sock_restart;
#endif
		return ret;
	}
	return sockd;
}

/* blocking receive */
/* Returns error number if it cannot get all len bytes
 * and the # of bytes received otherwise
 */
int brecv(int s, void *buf, int len)
{
	int oldfl, ret, comp = len;
	oldfl = Fcntl(s, F_GETFL, 0);
	if (oldfl & O_NONBLOCK) Fcntl(s, F_SETFL, oldfl & (~O_NONBLOCK));
	while (comp) {
#ifndef __KERNEL__
brecv_restart:
#endif
		if ((ret = Recv(s, (char *) buf, comp, 0)) < 0) {
#ifndef __KERNEL__
			if(errno == EINTR)
				goto bind_sock_restart;
#endif
			return ret;
		}
		if (!ret) {
			/* Note: this indicates a closed socket.  However, I don't
			 * like this behavior, so we're going to return -EPIPE
			 * instead.
			 */
			PERROR("Connection closed -EPIPE on socket %d\n",s);
			RETURN_ERROR(-EPIPE);
		}
		comp -= ret;
		buf = (char *) buf + ret;
	}
	return (len - comp);
}

/* nonblocking receive */
int nbrecv(int s, void *buf, int len)
{
	int oldfl, ret, comp = len;
	oldfl = Fcntl(s, F_GETFL, 0);
	if (!(oldfl & O_NONBLOCK)) Fcntl(s, F_SETFL, oldfl | O_NONBLOCK);
	while (comp) {
#ifndef __KERNEL__
nbrecv_restart:
#endif
		ret = Recv(s, buf, comp, 0);
		if (!ret) /* socket closed */ {
			PERROR("Connection closed, -EPIPE on socket %d\n",s);
			RETURN_ERROR(-EPIPE);
		}
#ifdef __KERNEL__
		if (ret == -EWOULDBLOCK) {
#else
		if (ret < 0 && errno = EWOULDBLOCK) {
#endif
			return (len - comp); /* return amount completed */
		}
		else if (ret < 0) {
#ifndef __KERNEL__
			if(errno == EINTR)
				goto nbrecv_restart;
#endif
			PERROR("error in nbrecv %d\n",ret);
			return ret;
		}
		comp -= ret;
		buf = (char *) buf + ret;
	}
	return (len - comp);
}

/* blocking send */
int bsend(int s, void *buf, int len)
{
	int oldfl, ret, comp = len;
	oldfl = Fcntl(s, F_GETFL, 0);
	if (oldfl & O_NONBLOCK) Fcntl(s, F_SETFL, oldfl & (~O_NONBLOCK));
	while (comp) {
#ifndef __KERNEL__
bsend_restart:
#endif
		if ((ret = Send(s, (char *) buf, comp, 0)) < 0) {
#ifndef __KERNEL__
			if(errno == EINTR) 
				goto bsend_restart;
#endif
			PERROR("error in send %d\n",ret);
			return ret;
		}
		comp -= ret;
		buf = (char *) buf + ret;
	}
	return (len - comp);
}

/* blocking vector send */
int bsendv(int s, const struct iovec *vec, int cnt)
{
	int tot, comp, ret, oldfl;
	struct iovec *cur = (struct iovec *)vec;
#ifdef BSENDV_NO_WRITEV
	char *buf;
#else
   struct iovec *alloced_iovecs = NULL;
#endif

	oldfl = Fcntl(s, F_GETFL, 0);
	if (oldfl & O_NONBLOCK) Fcntl(s, F_SETFL, oldfl & (~O_NONBLOCK));
#ifdef BSENDV_NO_WRITEV
	for (tot=0; cnt--; cur++) {
		buf = (char *) cur->iov_base;
		comp = cur->iov_len;
		while (comp) {
#ifndef __KERNEL__
bsendv_restart:
#endif
			if ((ret = Send(s, buf, comp, 0)) < 0) {
#ifndef __KERNEL__
				if(errno == EINTR)
					goto bsendv_restart;
#endif
				PERROR("bsendv_no_writev:Error in send %d\n",ret);
				return ret;
			}
			comp -= ret;
			buf = buf + ret;
		}
		tot += cur->iov_len;
	}
	return tot;
#else
	while (1) {
#ifndef __KERNEL__
bsendv_restart:
#endif
		if ((ret = Writev(s, cur, cnt)) < 0) {
#ifndef __KERNEL__
			if(errno == EINTR)
				goto bsendv_restart;
#endif
			PERROR("bsendv_yes_writev:Error in writev %d\n",ret);
			if (NULL != alloced_iovecs)
				release_memory(alloced_iovecs);
			return ret;
		}
		for(comp=0,tot=0; comp<cnt; comp++) {
			tot += cur[comp].iov_len;
			if (tot > ret)
				break;
		}
		if (tot == ret && comp == cnt) 
			break;
		if (vec == cur) {
			if (!(cur = (struct iovec *)allocate_memory(cnt*sizeof(struct iovec)))) {
				RETURN_ERROR(-ENOMEM);
			}
			alloced_iovecs = cur;
			memcpy(cur, vec, cnt*sizeof(struct iovec));
		}
		cnt -= comp;
		cur += comp;
		cur[0].iov_base = (char*)cur[0].iov_base + cur[0].iov_len-(tot-ret);
		cur[0].iov_len = tot-ret;
	}
	if (NULL != alloced_iovecs)
		release_memory(alloced_iovecs);
	return ret;
#endif
}

/* nonblocking send */
/* should always return 0 when nothing gets done! */
int nbsend(int s, void *buf, int len)
{
	int oldfl, ret, comp = len;
	oldfl = Fcntl(s, F_GETFL, 0);
	if (!(oldfl & O_NONBLOCK)) Fcntl(s, F_SETFL, oldfl | O_NONBLOCK);

	while (comp) {
#ifndef __KERNEL__
nbsend_restart:
#endif
		ret = Send(s, (char *) buf, comp, 0);
#ifdef __KERNEL__
		if (ret == 0 || (ret == -EWOULDBLOCK)) {
#else
	   if(ret == 0 || (ret < 0 && errno == EWOULDBLOCK))  {
#endif
			return (len - comp); /* return amount completed */
		}
		if (ret < 0) {
#ifndef __KERNEL__
			if(errno == EINTR)
				goto nbsend_restart;
#endif
			PERROR("Error in nbsend %d\n",ret);
			return ret;
		}
		comp -= ret;
		buf = (char *) buf + ret;
	}
	return (len - comp);
}

/* NBSENDFILE() - nonblocking (on the socket) send from file
 *
 * Here we are going to take advantage of the sendfile() call provided
 * in the linux 2.2 kernel to send from an open file directly (ie. w/out
 * explicitly reading into user space memory or memory mapping).
 *
 * We are going to set the non-block flag on the socket, but leave the
 * file as is.
 *
 * Returns error number, amount of data written to socket on success.
 */
int nbsendfile(int s, int f, int off, int len)
{
	int oldfl, ret, comp = len, myoff;

	oldfl = Fcntl(s, F_GETFL, 0);
	if (!(oldfl & O_NONBLOCK)) Fcntl(s, F_SETFL, oldfl | O_NONBLOCK);

	while (comp) {
#ifndef __KERNEL__
nbsendfile_restart:
#endif
		myoff = off;
		ret = Sendfile(s, f,(off_t*) &myoff, comp);
#ifdef __KERNEL__
		if (ret == 0 || (ret == -EWOULDBLOCK)) {
#else
	   if (ret == 0 || (ret < 0 && errno == EWOULDBLOCK)) {
#endif
			return (len - comp); /* return amount completed */
		}
		if (ret < 0) {
#ifndef __KERNEL__
			if(errno == EINTR)
				goto nbsendfile_restart;
#endif
			PERROR("nbsendfile error %d\n",ret);
			return ret;
		}
		comp -= ret;
		off += ret;
	}
	return (len - comp);
}

/* routines to get and set socket options */
int get_sockopt(int s, int optname)
{
	int val, len = sizeof(val), ret;
	if ((ret = Getsockopt(s, SOL_SOCKET, optname, &val, &len)) < 0) return ret;
	else return val;
}

int set_tcpopt(int s, int optname, int val)
{
	int ret;
	if ((ret = Setsockopt(s, SOL_TCP, optname, &val, sizeof(val))) < 0) {
		return ret;
	}
	else {
		return val;
	}
}

int set_sockopt(int s, int optname, int val)
{
	int ret;
	if ((ret = Setsockopt(s, SOL_SOCKET, optname, &val, sizeof(val))) < 0) {
		return ret;
	}
	else {
		return val;
	}
}

int get_socktime(int s, int optname)
{
	struct timeval val;
	int len = sizeof(val), ret ;
	if ((ret = Getsockopt(s, SOL_SOCKET, optname, &val, &len)) < 0) {
		return ret;
	}
	else {
		return (val.tv_sec * 1000000 + val.tv_usec);
	}
}

int set_socktime(int s, int optname, int size)
{
	int ret ;
	struct timeval val;
	val.tv_sec = size / 1000000;
	val.tv_usec = size % 1000000;
	if ((ret = Setsockopt(s, SOL_SOCKET, optname, &val, sizeof(val))) < 0) {
		return ret;
	}
	else {
		return size;
	}
}


/* connect_timeout()
 *
 * Attempts to do nonblocking connect() until a timeout occurs.  Useful for
 * recovering from deadlocks...
 */
int connect_timeout(int s, struct sockaddr *saddrp, int len, int time_secs)
{
	struct pollfd pfds;
	int ret, oldfl, val, val_len;

	/* set our socket to nonblocking */
	oldfl = Fcntl(s, F_GETFL, 0);
	if (!(oldfl & O_NONBLOCK)) Fcntl(s, F_SETFL, oldfl | O_NONBLOCK);

#ifndef __KERNEL__
connect_timeout_connect_restart:
#endif
	ret = Connect(s, saddrp, len);
	if (ret < 0) {
#ifndef __KERNEL__
		if(errno == EINTR)
			goto connect_timeout_connect_restart;
		if(errno != EINPROGRESS) {
#else
		if (ret != -EINPROGRESS) {
#endif
			  PERROR("Connect failed %d\n",ret);
			  goto connect_timeout_err;
		}
		pfds.fd = s;
		pfds.events = POLLOUT;
#ifndef __KERNEL__
connect_timeout_poll_restart:
#endif
		ret = Poll(&pfds, 1, time_secs * 1000);
		if (ret < 0) {
#ifndef __KERNEL__
			if(errno == EINTR)
				goto connect_timeout_poll_restart;
#endif
			PERROR("poll failed with %d\n",ret);
			goto connect_timeout_err;
		}
		if (ret == 0) {
#ifndef __KERNEL__
			err = errno;
#endif
			/* timed out */
			Close(s); /* don't want it to keep trying */
#ifndef __KERNEL__
			errno = err;
#endif
			ret = -1;
			goto connect_timeout_err;
		}
	}
	/* _apparent_ success -- check if connect really completed */
	val_len = sizeof(val);
	ret = Getsockopt(s, SOL_SOCKET, SO_ERROR, &val, &val_len);
	if (val != 0) {
#ifdef __KERNEL__
		ret = -val;
#else
		errno = val;
		ret = -1;
#endif
		PERROR("connect did fail though %d\n",ret);
		goto connect_timeout_err;
	}
	/* success -- make socket blocking again and return */
	Fcntl(s, F_SETFL, oldfl & (~O_NONBLOCK));
	return 0;
connect_timeout_err:
#ifndef __KERNEL__
	err = errno;
#endif
	/* set flags on socket back to what they were, return error number */
	Fcntl(s, F_SETFL, oldfl);
#ifndef __KERNEL__
	errno = err;
#endif
	return ret;
}


/* brecv_timeout()
 *
 * Attempts to do nonblocking recv's until a timeout occurs.  Useful for
 * recovering from deadlocks in situations where you want to give up if a
 * message does not finish arriving within a certain time frame.
 *
 * Same args as brecv, but with an additional timeout argument that is an
 * integer number of seconds.   
 * 
 * returns number of bytes received, even if it did not recv all that was
 * asked.  If any error other than timeout occurs, it returns 
 * error number .  */

int brecv_timeout(int s, void *vbuf, int len, int timeout)
{
	char *buf = (char *) vbuf;
	int recv_size = len;    /* amt we hope for each iteration */
	int recv_total = 0;     /* total amt recv'd thus far */
	char * recv_ptr = buf;   /* where to put the data */
	int initial_tries = 4;  /* number of initial attempts to make */
	int i = 0;
	int ret = -1;
	struct pollfd poll_conn;

	/* This determines the backoff times.  It will do a few attempts with the
	 * initial backoff first, then wait and do a final one after the
	 * remainder of the time is up.  Note that the initial backoff will be
	 * zero if a small overall timeout is specified by the caller.
	 */
	int initial_backoff = timeout/initial_tries;
	int final_backoff = timeout - (timeout/initial_tries);

	/* we don't accept -1 for infinite timeout.  That is the job of brecv. */
	if(timeout < 0) {
		RETURN_ERROR(-EINVAL);
	}

	/* initial attempts */
	do {
		if((ret = nbrecv(s, recv_ptr, recv_size)) < 0) {
			PERROR("Error in nbrecv %d\n",ret);
			/* bail out at any error */
			return ret;
		}

		/* update our progress */
		recv_size -= ret;
		recv_total += ret;
		recv_ptr += ret;

		/* if we didn't finish, then poll for a while. */
		if(recv_total < len) {
			poll_conn.fd = s;
			poll_conn.events = POLLIN;
#ifndef __KERNEL__
brecv_timeout_poll1_restart:
#endif
			ret = Poll(&poll_conn, 1, (initial_backoff*1000));
			if(ret < 0) {
#ifndef __KERNEL__
				if(errno == EINTR)
					goto brecv_timeout_poll1_restart;
#endif
				PERROR("Error in Poll %d\n",ret);
				/* bail out on any "real" error or signal*/
				return ret;
			}
		}
		i++;
		
	} while((i < initial_tries) && (recv_total < len));
	
	/* see if we are done */
	if (recv_total == len) {
		return len;
	}

	/* not done yet, give it one last chance: */
	poll_conn.fd = s;
	poll_conn.events = POLLIN;
#ifndef __KERNEL__
brecv_timeout_poll2_restart:
#endif
	ret = Poll(&poll_conn, 1, (final_backoff*1000));
	if(ret < 0) {
#ifndef __KERNEL__
		if(errno == EINTR)
			goto brecv_timeout_poll2_restart;
#endif
		PERROR("Error in Poll %d\n",ret);
		/* bail out on any "real" error or signal */
		return ret;
	}
	if((ret = nbrecv(s, recv_ptr, recv_size)) < 0) {
		PERROR("brecv_timeout: giving up with error %d",ret);
		return ret;
	}
	recv_size -= ret;
	recv_total += ret;
	return recv_total;
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 * End:
 *
 * vim: ts=3
 */
