/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */

#include "wrapper.h"
#include "sockset.h"

/* FUNCTIONS */
void addsockrd(int sock, sockset_p set_p)
{
	dfd_set(sock, &set_p->read);
	/* max_index MUST equal (max. sock # + 1) */
	if (sock >= set_p->max_index) 
			set_p->max_index = sock+1;
	return;
}

void addsockwr(int sock, sockset_p set_p)
{
	dfd_set(sock, &set_p->write);
	if (sock >= set_p->max_index) 
			set_p->max_index = sock+1;
	return;
}

void delsock(int sock, sockset_p set_p)
{
	/* make sure we're not going to check for this any more */
	dfd_clr(sock, &set_p->read);
	dfd_clr(sock, &set_p->write);
	dfd_clr(sock, &set_p->tmp_read);
	dfd_clr(sock, &set_p->tmp_write);

	if (sock + 1 >= set_p->max_index) /* need to reduce max_index */ {
		int i;
		for (i = sock; i >= 0; i--) {
			if (dfd_isset(i, &set_p->read) || dfd_isset(i, &set_p->write)) 
					break;
		}
		set_p->max_index = i+1;
	}
	return;
}

/* nextsock() - find a ready but unserviced socket in our sockset
 * Marks socket as "serviced" before returning, so it won't be returned
 * again until checksock() is used again and it is shown to be ready.
 * Returns socket number on success, -1 on failure.
 */
int nextsock(sockset_p set_p)
{
	int i = 0, startval = 0;
	int max_index;

	max_index = set_p->max_index;
	if (max_index <= 0) {
		RETURN_ERROR(-EINVAL);
	}
	/* we want to avoid using two loops in the normal case, so we check
	 * the entire range with this last one.
	 */
	startval = max_index;
	for (i = 0; i < startval; i++) {
		if (dfd_isset(i, &set_p->tmp_read) || dfd_isset(i, &set_p->tmp_write)) {
			dfd_clr(i, &set_p->tmp_read);
			dfd_clr(i, &set_p->tmp_write);
			return i;
		}
	}
	return -1;
} /* end of nextsock() */


int dumpsocks(sockset_p set_p)
{
	int i;
	PERROR("max_index = %d\n", set_p->max_index);
	for (i = 0; i < set_p->max_index; i++) {
		if (dfd_isset(i, &set_p->read)) PERROR("%d:\tREAD\n", i);
		if (dfd_isset(i, &set_p->write)) PERROR("%d:\tWRITE\n", i);
	} 
	return(0);
} /* end of DUMPSOCKS() -- select() version */


void finalize_set(sockset_p set_p)
{
	dfd_finalize(&set_p->read);
	dfd_finalize(&set_p->write);
	dfd_finalize(&set_p->tmp_read);
	dfd_finalize(&set_p->tmp_write);
	return;
}

void initset(sockset_p set_p)
{
	set_p->max_index = 0;
	/* call dfd_init() in case the sets have never been initialized;
	 * if that fails then call dfd_zero() to clean everything out
	 * FIXME: Arent we having memory leaks here?????
	 */
	if (dfd_init(&set_p->read, 0) < 0) {
		dfd_zero(&set_p->read);
	}
	if (dfd_init(&set_p->write, 0) < 0) {
		dfd_zero(&set_p->write);
	}
	if (dfd_init(&set_p->tmp_read, 0) < 0) {
		dfd_zero(&set_p->tmp_read);
	}
	if (dfd_init(&set_p->tmp_write, 0) < 0) {
		dfd_zero(&set_p->tmp_write);
	}
	return;
}

/* time given in msec */
/* Returns # of sockets in descriptor set (?), or -1 on error
 */
int check_socks(sockset_p set_p, int time)
{
	int ret = 0;
	/* copy the real structures into the temporary ones */
#ifndef __KERNEL__
check_socks_restart:
#endif
	dfd_copy(&set_p->tmp_read, &set_p->read);
	dfd_copy(&set_p->tmp_write, &set_p->write);

	if (time == -1) {
		ret = dfd_select(set_p->max_index, &set_p->tmp_read, &set_p->tmp_write, NULL, NULL);
	}
	else {
		set_p->timeout.tv_sec = time / 1000;
		set_p->timeout.tv_usec = (time % 1000) * 1000;
		ret = dfd_select(set_p->max_index, &set_p->tmp_read, &set_p->tmp_write, NULL, &set_p->timeout);
	}
#ifndef __KERNEL__
	if (ret < 0 && errno == EINTR) goto check_socks_restart;
#endif
	return(ret);
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 * End:
 *
 * vim: ts=3
 */
